// Copyright 2000-2022 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.intellij.workspaceModel.codegen.deft.meta

interface Obj {
  val objType: ObjType<*>
}

/**
 * Mutable, possibly not fully initialized [Obj].
 *
 * Implementation of that interface is generated by the Deft compiler,
 * inside each [Obj]. It will contain `var`s for all final fields.
 *
 * [ObjBuilder] inherits all corresponding builders of [Obj] supertypes.
 *
 * Doesn't have `var`s for open field, as open setters contravariance
 * contradicts open getter invariance. [ObjFinalBuilder] contains rest fields,
 * but **not** inherits supertypes Builders.
 */
interface ObjBuilder<T : Obj> : Obj {
  val isInitialized: Boolean

  fun isInitialized(field: OwnProperty<T, *>): Boolean
  fun isInitialized(field: ExtProperty<T, *>): Boolean

  fun <V> setValue(field: OwnProperty<T, V>, value: V)
  fun <V> setValue(field: ExtProperty<T, V>, value: V)
  fun <V : Obj> setValue(type: ObjClass<V>, value: V)

  fun remove(field: ExtProperty<T, *>): Boolean
  fun remove(type: ObjClass<*>): Boolean

  fun build(): T
}

/**
 * See [ObjBuilder].
 *
 * Contains rest open fields, and not inherits supertypes Builders
 * (as open setters contravariance contradicts open getter invariance).
 */
interface ObjFinalBuilder<T : Obj> : ObjBuilder<T>

